@@ -105,24 +105,17 @@ class Parser(object):
                 # Set the next active variable
                 if self.token in ('=', '(', '%'):
 
-                    v_name, v_values = self.parse_variable(patch_nml=grp_patch)
-
+                    v_name, v_values = self.parse_variable(g_vars,
+                                                           patch_nml=grp_patch)
                     if v_name in g_vars:
                         v_prior_values = g_vars[v_name]
-                        if not type(v_prior_values) is list:
-                            v_prior_values = [v_prior_values]
-
                         v_values = merge_values(v_prior_values, v_values)
 
-                    if len(v_values) == 0:
-                        v_values = None
-                    elif len(v_values) == 1:
-                        v_values = v_values[0]
-
                     if v_name in g_vars and type(g_vars[v_name]) is NmlDict:
                         g_vars[v_name].update(v_values)
                     else:
                         g_vars[v_name] = v_values
+
                     # Deselect variable
                     v_name = None
                     v_values = []
@@ -167,7 +160,7 @@ class Parser(object):
         return nmls
 
 
-    def parse_variable(self, patch_nml={}):
+    def parse_variable(self, parent, patch_nml={}):
         """Parse a variable and return its name and values."""
 
         v_name = self.prior_token
@@ -197,19 +190,17 @@ class Parser(object):
 
             # Resolve the derived type
 
+            v_parent = parent[v_name] if parent and v_name in parent else []
+
             self.update_tokens()
             self.update_tokens()
-            v_att, v_att_vals = self.parse_variable()
+            v_att, v_att_vals = self.parse_variable(v_parent)
 
-            # TODO: resolve indices
-            next_value = NmlDict()
-
-            if len(v_att_vals) == 0:
-                v_att_vals = None
-            elif len(v_att_vals) == 1:
-                v_att_vals = v_att_vals[0]
-
-            next_value[v_att] = v_att_vals
+            if v_idx and v_att in parent:
+                next_value = v_att_vals
+            else:
+                next_value = NmlDict()
+                next_value[v_att] = v_att_vals
 
             append_value(v_values, next_value, v_idx)
 
@@ -290,9 +281,9 @@ class Parser(object):
                     ws_sep = self.update_tokens(write_token)
 
         if patch_values:
-            return v_name, patch_values
+            return v_name, delist(patch_values)
         else:
-            return v_name, v_values
+            return v_name, delist(v_values)
 
 
     def parse_index(self):
@@ -446,6 +437,19 @@ def append_value(v_values, next_value, v_idx=None, n_vals=1):
 
 
 def merge_values(src, new):
+
+    if isinstance(src, dict) and isinstance(new, dict):
+        return merge_dicts(src, new)
+    else:
+        if not isinstance(src, list):
+            src = [src]
+        if not isinstance(new, list):
+            new = [new]
+
+        return merge_lists(src, new)
+
+
+def merge_lists(src, new):
     """Update a value list with a list of new or updated values."""
 
     l_min, l_max = (src, new) if len(src) < len(new) else (new, src)
@@ -453,6 +457,38 @@ def merge_values(src, new):
     l_min.extend(None for i in range(len(l_min), len(l_max)))
 
     for i, val in enumerate(new):
-        new[i] = val if val else src[i]
+        if isinstance(val, dict) and isinstance(src[i], dict):
+            new[i] = merge_dicts(src[i], val)
+        elif val is not None:
+            new[i] = val
+        else:
+            new[i] = src[i]
 
     return new
+
+
+def merge_dicts(src, patch):
+    """Merge contents of dict `patch` into `src`."""
+
+    for key in patch:
+        if key in src:
+            if isinstance(src[key], dict) and isinstance(patch[key], dict):
+                merge_dicts(src[key], patch[key])
+            else:
+                src[key] = merge_values(src[key], patch[key])
+        else:
+            src[key] = patch[key]
+
+    return src
+
+
+def delist(values):
+    """Reduce lists of zero or one elements to individual values."""
+    assert isinstance(values, list)
+
+    if not values:
+        return None
+    elif len(values) == 1:
+        return values[0]
+    else:
+        return values
