@@ -36,6 +36,7 @@ class Test(unittest.TestCase):
                              'v_null_start': [None, 2, 3, 4],
                              'v_null_interior': [1, 2, None, 4],
                              'v_null_end': [1, 2, 3, None],
+                             'v_zero': [1, 0, 3],
                             }
                           }
 
@@ -73,6 +74,23 @@ class Test(unittest.TestCase):
                                     'y': 2,
                                     'z': 3,
                                    }
+                            },
+                          'dtype_nested_nml':
+                            {'f': {'g': {'x': 1,
+                                         'y': 2,
+                                         'z': 3,
+                                         }
+                                  }
+                            },
+                          'dtype_field_idx_nml':
+                            {'f': {'x': [1, 2, 3]}
+                            },
+                          'dtype_vec_nml':
+                            {'a': {'b': [{'c': 1, 'd': 2},
+                                         {'c': 3, 'd': 4},
+                                         {'c': 5, 'd': 6}
+                                        ]
+                                  }
                             }
                          }
 
@@ -144,7 +162,7 @@ class Test(unittest.TestCase):
     def test_dtype(self):
         test_nml = f90nml.read('dtype.nml')
         self.assertEqual(self.dtype_nml, test_nml)
-        self.assert_write(test_nml, 'dtype.nml')
+        self.assert_write(test_nml, 'dtype_target.nml')
 
     def test_bcast(self):
         test_nml = f90nml.read('bcast.nml')
@@ -186,4 +204,6 @@ class Test(unittest.TestCase):
 
 
 if __name__ == '__main__':
+    if os.path.isfile('tmp.nml'):
+        os.remove('tmp.nml')
     unittest.main()
